#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/iocan128.h>

#include "one_wire.h"
#include "one_wire_adc.h"
#include "one_wire_dualSwitch.h"
#include "one_wire_simpleSwitch.h"
#include "read_write_register.h"
#include "one_wire_temperature.h"

#include "adc.h"
#include "api.h"
#include "api_define.h"
#include "api_global.h"
#include "can.h"
#include "mem-check.h"

#include "OWIDeviceSpecific.h"
#include "OWIHighLevelFunctions.h"
#include "OWIBitFunctions.h"


char buffer[5];/*variable to store keyword* "RECV"*/
volatile uint16_t thigh = 900; /*the maximum gas pressure*/
volatile uint16_t tlow = 100;  /*the minimum gas pressure*/
uint16_t Voltages[8];          /* current supply voltages of the board*/
uint8_t res0, res1, res2, res3;/*parameter back to reading the voltage on ADC-channel*/

uint8_t IDS_ALL[NUM_DEVICES][8];  /*Global variable to store the ID numbers of ALL devices*/
uint16_t IDS_ALL_VALUES[NUM_DEVICES][8];  /*Global variable to store the register values of ALL devices*/
uint8_t NumDevicesFound;
uint8_t BUSES[5] = { OWI_PIN_0 };

char PD;/*variable to check found devices*/
int8_t countDEV = 0;/*number of devices (initialized to 0)*/
int8_t countDEVbus = 0;/*number of devices on the current bus (initialized to 0)*/


enum states {idle, below_low_thres, in_between , above_high_thres };
uint8_t cur_state;
uint8_t usercommand;
/*variable to store values*/
//char tmp_str[BUFFER_SIZE];
char* ptr_setParameter[MAX_PARAMETER];

volatile unsigned char timer0Ready;/*variable for Timer0  Interrupt*/
volatile unsigned char timer1Ready;/*variable for Timer1  Interrupt*/
volatile unsigned char timer0AReady;/*variable for Timer0 Output Compare A  Interrupt*/
volatile unsigned char timer0ASchedulerReady;/*variable for Timer0 Output Compare A  Interrupt*/
volatile unsigned char canReady;/*variable for can interrupt*/
volatile unsigned char uartReady;/*variable for Uart interrupt*/
char schedulerString[5]= "OWTP";/*only for scheduler*/
char keepAliveString[15]= "PING";/*only for keep_alive function*/
char uartString[BUFFER_SIZE]; /* variable for storage received a complete string via UART */
char decrypt_uartString[BUFFER_SIZE];
char tempstr[BUFFER_SIZE];/*variable for debug*/
char ring_buffer[MAX_INPUT][MAX_LENGTH_CAN_DATA]; /*buffer to store CAN data*/
char setParameter[MAX_PARAMETER][MAX_LENGTH_PARAMETER];
char canString[MAX_LENGTH_CAN_DATA];
unsigned char nextCharPos; /*pointer of the variable uartString  */
uint8_t can_errorCode; /* error code for CAN-communication */
uint8_t uart_errorCode; /* error code for UART-communication */
uint8_t mailbox_errorCode; /* error code for Message Object Block */
uint8_t general_errorCode; /*  general error code */
uint8_t ptr_subscribe;/* pointer of variable subscribe_ID and subscribe_mask */
uint8_t mob;  /*variable  for Message Object Block in the interrupt routine*/
int8_t can_init; /* variable to call CAN_Init function*/
int8_t timer0_init; /* variable to call Timer0_Init function*/
int8_t timer0A_init;/* variable to call Timer0A_Init function*/
uint8_t ptr_buffer_in; /*pointer to write CAN data in buffer_ring*/
uint8_t ptr_buffer_out; /*pointer to read CAN data in buffer_ring*/
uint8_t flag_pingActive = 0; /*automatic ping action disabled*/

/*define UART-structur */
struct uartStruct uartFrame;
struct uartStruct *ptr_uartStruct;
/*define CAN-structur */
struct canStruct canFrame;
struct canStruct *ptr_canStruct;

int main (void) {
	ptr_uartStruct = &uartFrame;  /* initialize pointer for CPU-struct */
	ptr_canStruct = &canFrame;  /* initialize pointer for CAN-struct */
	/* initialize flag */
	nextCharPos = 0;
	uartReady = 0;
	canReady = 0;
	timer0Ready = 0;
	timer1Ready = 0;
	timer0AReady = 0;
	timer0ASchedulerReady = 0;
	ptr_subscribe = 2; /*start value of pointer*/
	ptr_buffer_in = 0; /*initialize pointer for write in buffer_ring*/
	ptr_buffer_out = 0;  /*initialize pointer for read in  buffer_ring*/
	res0 = 0, res1 = 0, res2 = 0, res3 = 0;

	uint8_t i;
	cur_state = idle;

	cli();  /* disable interrupts*/
	UART0_Init();
	Init_Port ();
	can_init = CAN_Init(TWOHUNDERTFIFTY_KPS);   /* initialize can-contoller with a baudrate 250kps */



    // init section

	if(-1 == can_init)
	{
		can_errorCode = Communication_Error(can_error[8],can_error_number[8], "ERRC");
		exit(0);
	}
	else
	{
		timer0_init =Timer0_Init(); /* initialize first timer */

	}
	if( 1 != timer0_init )
	{
		general_errorCode = Communication_Error(general_error[0],general_error_number[0], "ERRG");
		exit(0);
	}
	else
	{
		timer0A_init = Timer0A_Init(); /* initialize second timer*/

	}
	if(1!= timer0A_init)
	{
		general_errorCode = Communication_Error(general_error[1],general_error_number[1], "ERRG");
		exit(0);
	}


	sei();  /* enable interrupts*/

#warning where else do we talk about pressure and low and high thresholds ????
#warning TODO: put this into an function threshold_init

	// start threshold block
	/*pressure is checked only for channel 0*/
	res0 = GetVoltage (0);
	PORTA = PORTA | (0x2); // turn on lamp

	if(Voltages[0] < tlow) {
		cur_state = below_low_thres;
		PORTA = PORTA & (~0x1); // turn off relais
	}
	else if(Voltages[0] > thigh) {
		cur_state = above_high_thres;
		PORTA = PORTA & (~0x1); // turn off relais
	}
	else {
		cur_state = in_between;
		PORTA = PORTA | (0x1); // turn on relais
	}
	// end threshold block

	// main endless loop
	while(1)
	{
		// UART has received a string
		if( 1 == uartReady )/* of the ISR was completely receive a string */
		{
			cli(); // disable interrupts

			strncpy( decrypt_uartString, uartString, BUFFER_SIZE-1);
			//  			snprintf(tempstr,BUFFER_SIZE-1,"command:%s",decrypt_uartString);
			//  			UART0_Send_Message_String(tempstr);
			memset(uartString,0,BUFFER_SIZE-1);
			Decrypt_Uart_String();
			uartReady = 0; /* restore flag */

			sei(); // (re)enable interrupts
		}

		// CANbus interface has received a message
		if( 1 == canReady  ) /* of the ISR data has been fully received */
		{
			cli(); // disable interrupts

			Convert_pack_canFrame_to_UartFormat(ptr_canStruct);
			canReady = 0; /* restore flag */

			sei(); // (re)enable interrupts
		}

		// timer 0 set by ISR
		if( 1 == timer0Ready )
		{
			cli();
			Get_BusState();
			timer0Ready = 0;
			sei();
		}

		// timer 1 set by ISR


 		if(1 == timer1Ready )
 		{
			if (flag_pingActive)
			{
				cli(); // disable interrupts
				strncpy(tempstr,keepAliveString, BUFFER_SIZE-1);
                UART0_Send_Message_String(tempstr);

 		        timer1Ready = 0;
				sei();
			}
 		}



		// timer 0A set by ISR
		if(1 == timer0AReady )
		{
			cli();
			Get_CanError();
			timer0AReady =0; /* restore flag */
			sei();
		}

// 		if(1 == timer0ASchedulerReady )
// 		{
// 			cli();
// 			strncpy(decrypt_uartString,schedulerString, BUFFER_SIZE-1);
// 			Decrypt_Uart_String();
// 			timer0ASchedulerReady = 0; /* restore flag */
// 			sei();
// 		}

		res0 = GetVoltage (0);

		switch (cur_state) {
			case in_between:
			{

				if(Voltages[0] > thigh) {

					/*turn off relais*/
					PORTA = PORTA & (~0x1);
					cur_state = above_high_thres;
				}
				if(Voltages[0] < tlow) {
					/*turn off relais*/
					PORTA = PORTA & (~0x1);
					cur_state = below_low_thres;
				}
			}
			break;
			case below_low_thres:
			{
				_delay_ms(100);
				_delay_ms(100);
				PINA = 0x2; // toggle LED

				if(Voltages[0] > tlow +2) {
					/*turn on relais*/
					PORTA = PORTA | (0x3);
					cur_state = in_between;
				}
			}
			break;
			case above_high_thres:
			{

				_delay_ms(100);
				_delay_ms(100);
				PINA = 0x2; // toggle LED


				if(Voltages[0] < thigh -2) {
					/* turn on relais and lamp*/
					PORTA = PORTA | (0x3);
					cur_state = in_between;
				}
			}
			break;
			default:
			{
				cur_state = below_low_thres;

			}
		}

#warning what is this needed for ???
		i = i*2;


	}// END of while


} // END of main
