/*
 * VERSION 1.0 Januar 7th 2010 LATE  File: 'one_wire_adc.c'
 * Author: Linda Fouedjio  based on Alejandro Gil
 */

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include <avr/iocan128.h>

#include "one_wire.h"
#include "one_wire_adc.h"
#include "one_wire_dualSwitch.h"
#include "one_wire_simpleSwitch.h"
#include "read_write_register.h"
#include "one_wire_temperature.h"

#include "adc.h"
#include "api.h"
#include "api_define.h"
#include "api_global.h"
#include "can.h"
#include "mem-check.h"

#include "OWIDeviceSpecific.h"
#include "OWIHighLevelFunctions.h"
#include "OWIBitFunctions.h"

#define DS2450_CONVERT                  0x3C
#define DS2450_READ_MEMORY              0xAA

/*
 * this function contains all the functions that are necessary to
 * convert analog value to digital via one wire bus
 */

void read_WADC(struct uartStruct *ptr_uartStruct) {

	NumDevicesFound = ReadDevicesID(BUSES, 0);
	if (0 < NumDevicesFound) {
		ADCsInitialization(BUSES);
		MakeADCConversionOfAllDevices(BUSES);
		FindFamilyDevicesAndGetValues(BUSES, NumDevicesFound, FAMILY_DS2450_ADC);
	}

}//END of read_WADC function




/*
 *this function initializes the analog to digital converter
 */
int8_t ADCsInitialization(uint8_t *pins) {

	uint16_t receive_CRC;

	/*SET UP non parasitic mode*/
	for (int8_t b = 0; b < PIN_BUS + 1; b++) { //OWI_Init(pins[b]);
		if (0 == OWI_DetectPresence(pins[b])) { /* the "DetectPresence" function already sends a Reset*/

			return -99; // Error

		}

		OWI_SendByte(OWI_ROM_SKIP, pins[b]); //
		OWI_SendByte(0x55, pins[b]); //write memory
		OWI_SendByte(0x1C, pins[b]); //address for VCC powered mode
		OWI_SendByte(0x00, pins[b]);
		OWI_SendByte(0x40, pins[b]); /*data for VCC powered mode*/
		receive_CRC = OWI_ReceiveByte(pins[b]); /* IMPORTANT AFTER EACH 'MEMORY WRITE' OPERATION*/
		receive_CRC |= (OWI_ReceiveByte(pins[b]) << 8); /* IMPORTANT AFTER EACH 'MEMORY WRITE' OPERATION*/

		OWI_DetectPresence(pins[b]); /*the "DetectPresence" function already sends a Reset*/
		OWI_SendByte(OWI_ROM_SKIP, pins[b]);
		OWI_SendByte(0x55, pins[b]); /*write memory*/
		OWI_SendByte(0x09, pins[b]); /*address for range change*/
		OWI_SendByte(0x00, pins[b]);
		OWI_SendByte(0x8D, pins[b]); /* RANGE: 0x8C-->2.55V / 0x8D-->5V // C=12=1100 / D=13=1101*/
		receive_CRC = OWI_ReceiveByte(pins[b]); /*IMPORTANT AFTER EACH 'MEMORY WRITE' OPERATION*/
		receive_CRC |= (OWI_ReceiveByte(pins[b]) << 8); /*IMPORTANT AFTER EACH 'MEMORY WRITE' OPERATION*/

	}

	return 1;
}//END of ADCsInitialization function


/*
 *this function makes ADC converstaion o all find devices
 */
int8_t MakeADCConversionOfAllDevices(uint8_t *pins) {

	uint16_t receive_CRC;
	int8_t value;

	for (int8_t b = 0; b < PIN_BUS + 1; b++) {

		if (0 == OWI_DetectPresence(pins[b])) { /*the "DetectPresence" function already sends a Reset*/
			return -99; /*Error*/

		}

		OWI_SendByte(OWI_ROM_SKIP, pins[b]);
		OWI_SendByte(DS2450_CONVERT, pins[b]); /*conversion*/
		OWI_SendByte(0x0F, pins[b]); /* select mask*/
		OWI_SendByte(0xAA, pins[b]); /*select read out control*/
		receive_CRC = OWI_ReceiveByte(pins[b]); /*IMPORTANT AFTER EACH 'MEMORY WRITE' OPERATION*/
		receive_CRC |= (OWI_ReceiveByte(pins[b]) << 8); /*IMPORTANT AFTER EACH 'MEMORY WRITE' OPERATION*/

		value = 0;
		while (1 != value) //bucle that waits for the conversion to be done
		{
			value = OWI_ReadBit(pins[b]);
			_delay_ms(10);
		}
	}//end of for(int b=0;b<PIN_BUS+1;b++)


	return 1;
}//END of MakeADCConversionOfAllDevices function


/*
 *this function reads the state of dual switch
 */
int8_t ReadDualSwitches(unsigned char bus, unsigned char * id,
		uint8_t *array_chn) {

	if (0 == OWI_DetectPresence(bus)) {

		return -99; /* Error*/
	}

	OWI_MatchRom(id, bus); /* Match id found earlier*/
	OWI_SendByte(0xF5, bus); //PIO Access read command ich muss es überprüfen //???
	//OWI_SendByte(0xFF5, bus); //PIO Access read command //???

	/*Read two first bytes and place them in the 16 bit channel variable*/
	array_chn[0] = OWI_ReceiveByte(bus);
	array_chn[0] |= (OWI_ReceiveByte(bus) << 8);

	return 1;
}//END of ReadDualSwitches function

/*
 *this function
 */

int8_t ReadSimpleSwitches(unsigned char bus, unsigned char * id,
		uint8_t *array_chn) {

	if (0 == OWI_DetectPresence(bus)) {

		return -99; /* Error*/
	}

	OWI_MatchRom(id, bus); /* Match id found earlier*/
	OWI_SendByte(0xF5, bus); //PIO Access read command //???
	//OWI_SendByte(0xFF5, bus); //PIO Access read command //???

	/*Read two first bytes and place them in the 16 bit channel variable*/
	array_chn[0] = OWI_ReceiveByte(bus);
	array_chn[0] |= (OWI_ReceiveByte(bus) << 8);

	return 1;
}//END of ReadSimpleSwitches function


/*
 *this function gets the ADC-value of all channel 
 */
int8_t ReadADCchannels(unsigned char bus, unsigned char * id,
		uint16_t *array_chn, const int8_t size) {

	if (0 == OWI_DetectPresence(bus)) {

		return -99; // Error

	}

	OWI_MatchRom(id, bus); // Match id founded earlier
	OWI_SendByte(DS2450_READ_MEMORY, bus);
	OWI_SendByte(0x00, bus); //Send two bytes address (ie: 0x00 & 0x00,0x08 & 0x00,0x10 & 0x00,0x18 & 0x00)
	OWI_SendByte(0x00, bus);

	// Read two first bytes and place them in the 16 bit channel variable.
	array_chn[0] = OWI_ReceiveByte(bus);
	array_chn[0] |= (OWI_ReceiveByte(bus) << 8);

	array_chn[1] = OWI_ReceiveByte(bus);
	array_chn[1] |= (OWI_ReceiveByte(bus) << 8);

	array_chn[2] = OWI_ReceiveByte(bus);
	array_chn[2] |= (OWI_ReceiveByte(bus) << 8);

	array_chn[3] = OWI_ReceiveByte(bus);
	array_chn[3] |= (OWI_ReceiveByte(bus) << 8);

	return 1;
}//END of ReadADCchannels function


